#!/usr/bin/env python
#-*- coding:utf-8 -*-

from gi.repository import Gtk, Gio, GLib, GtkSource
import ui

MESSAGEDLG_TITLE = "memopolix"
VERSION = "0.0.0"

class TreeWin(Gtk.Window):
    """
        GtkTreeView Sample
    """
    def __init__(self):
        Gtk.Window.__init__(self)
        # GtkUIManager and GtkAccelGroup
        self.uimanager = Gtk.UIManager()
        accelgroup = self.uimanager.get_accel_group()
        self.add_accel_group(accelgroup)
        # GtkActionGroup
        self.actiongroup = Gtk.ActionGroup("memopolix_menu")
        self.actiongroup.add_actions(ui.get_action(self))
        self.uimanager.insert_action_group(self.actiongroup, 0)
        self.uimanager.add_ui_from_string(ui.menu_xml)
        # Menubar and Toolbar
        menubar = self.uimanager.get_widget("/MenuBar")
        self.statusbar = Gtk.Statusbar()
        toolbar = self.uimanager.get_widget("/ToolBar")
        toolbar.set_style(Gtk.ToolbarStyle.ICONS)
        style = toolbar.get_style_context()
        Gtk.StyleContext.add_class(style, Gtk.STYLE_CLASS_PRIMARY_TOOLBAR)
        #
        # CellRenderer
        cell = Gtk.CellRendererText()
        column = Gtk.TreeViewColumn("Tree Title", cell)
        column.add_attribute(cell, "text", 0)
        # TreeView and TreeStore
        self.store = Gtk.TreeStore.new([str, str])
        tree = Gtk.TreeView.new_with_model(self.store)
        tree.append_column(column)
        # Hide Header
        tree.props.headers_visible = False
        # Signal
        selection = tree.get_selection()
        selection.connect("changed", self.on_selection_changeed)
        #
        # Read
        filename = "{0}/{1}".format(GLib.get_home_dir(), ".config/sasakima/memopoli")
        gfile = Gio.file_new_for_path(filename)
        if gfile.query_exists(None):
            self.read_file(self.store, gfile)
        else:
            filename = "{0}/{1}".format(GLib.get_home_dir(), ".gjotsfile")
            gfile = Gio.file_new_for_path(filename)
            r = self.messagebox("import .gjotsfile ?", Gtk.MessageType.QUESTION, Gtk.ButtonsType.YES_NO)
            if r == Gtk.ResponseType.YES:
                self.read_file(self.store, gfile)
        #
        # GtkSourceView
        self.view = GtkSource.View()
        self.view.set_show_line_numbers(True)
        # Paned and ScrolledWindow
        swa = Gtk.ScrolledWindow()
        swa.add(tree)
        swb = Gtk.ScrolledWindow()
        swb.add(self.view)
        hp = Gtk.HPaned()
        hp.add1(swa)
        hp.add2(swb)
        vbox = Gtk.VBox()
        vbox.pack_start(menubar, False, True, 0)
        vbox.pack_start(toolbar, False, True, 0)
        vbox.pack_start(hp, True, True, 0)
        vbox.pack_start(self.statusbar, False, True, 0)
        
        # self
        self.connect("destroy", self.on_destroy)
        self.set_title("memopolix")
        self.resize(640, 480)
        hp.set_position(300)
        self.add(vbox)
        self.show_all()

#
# Function
#

    def messagebox(self, text, icon=Gtk.MessageType.INFO, button=Gtk.ButtonsType.OK):
        """
            icon @ INFO, WARNING, QUESTION, ERROR, OTHER
            button @ NONE, OK, CLOSE, CANCEL, YES_NO, OK_CANCEL
        """
        dlg = Gtk.MessageDialog(
                self,
                Gtk.DialogFlags.MODAL,
                icon,
                button,
                text)
        dlg.set_title(MESSAGEDLG_TITLE)  
        r = dlg.run()  
        dlg.destroy()
        return r

    def read_file(self, store, gfile):
        """
            Read ~/.config/sasakima/memopoli
            or ~/.gjotsfile
        """
        t = ".gjotsfile"
        s = ""
        it = None
        itold = None
        fstream = gfile.read(None)
        try:
            dstream = Gio.DataInputStream.new(fstream)
            while 1:
                line, length = dstream.read_line_utf8(None)
                if line == None:
                    break
                elif line == "\\NewEntry":
                    if t == ".gjotsfile":
                        it = store.append(it, [t, s])
                    elif t != "\\NoneTitle":
                        store.append(it, [t, s])
                    s = ""
                    t = "\\NoneTitle"
                elif line == "\\NewFolder":
                    itold = it.copy()
                    it = store.append(it, [t, s])
                    s = ""
                    t = "\\NoneTitle"
                elif line == "\\EndFolder":
                    store.append(it, [t, s])
                    it = itold
                    s = ""
                    t = "\\NoneTitle"
                else:
                    if t == "\\NoneTitle":
                        t = line
                    s += "{0}\n".format(line)
        finally:
            fstream.close(None)

    def tree_all_text(self, store, it, first):
        if not first:
            self.result += "\\NewEntry\n"
        body = store.get_value(it, 1)
        self.result += body
        if body and not body[-1] == "\n":
            self.result += "\n"
        it_child = store.iter_nth_child(it, 0)
        if it_child:
            if not first:
                self.result += "\\NewFolder\n"
            while it_child:
                self.tree_all_text(store, it_child, 0)
                it_child = store.iter_next(it_child)
            if not first:
                self.result += "\\EndFolder\n"

    def write_file(self, store):
        self.result = ""
        it = store.get_iter_first()
        if it:
            self.tree_all_text(store, it, 1)
        f = open("test.txt", "w")
        f.write(self.result)
        f.close()


#
# Signal
#

    def on_new(self, widget, data=None):
        pass

    def on_open(self, widget, data=None):
        pass

    def on_save(self, widget, data=None):
        self.write_file(self.store)

    def on_save_as(self, widget, data=None):
        pass

    def on_quit(self, widget, data=None):
        self.emit("destroy")

    def on_move(self, widget, data=None):
        """
            widget @ GtkAction
        """
        s = widget.get_name()
        if s == "up":
            pass

    def on_help(self, widget, data=None):
        pass

    def on_about(self, widget, data=None):
        pass

    def on_destroy(self, widget, data=None):
        print "Bye!"

    def on_selection_changeed(self, widget, data=None):
        """
            Params @ widget: GtkTreeSelection
        """
        model, it = widget.get_selected()
        if it:
            buf = self.view.get_buffer()
            buf.set_text(model.get_value(it, 1))

#
# Application
#

class App(Gtk.Application):
    def __init__(self):
        Gtk.Application.__init__(
                self,
                application_id="apps.test.treeview",
                flags=Gio.ApplicationFlags.FLAGS_NONE)
        self.connect("activate", self.on_activate)
        
    def on_activate(self, data=None):
        w = TreeWin()
        w.set_application(self)
    
if __name__ == "__main__":
    app = App()
    app.run(None)
